/*****************************************************
               hbsaverage.c, interpol.c
            Interpolation ( Lagrange's method )

 Subroutine of Calculation average value for HBS reduction

             K.Kawabata    1995.08.19
             K.Kawabata    1998.06.10. for hbsiqupt.c
             M.Isogai      2006.02.10  for AndorCCD

interpol()
	     
arguments          discription

  n_tbl             dimension of tbl_x and tbl_y
 tbl_x[n], tbl_y[n] (n=0,1,2,...,n_tbl)  reference table
                    note: tbl_x[] must be ascending or discending sorted
   x                interpolation parameter
   order            order of interpolation ( 1 < order < 10 )

return value        y(x): interpolated value


hbsaverage()

arguments          discription

  n_tbl             dimension of tbl_x and tbl_y
 tbl_x[n], tbl_y[n] (n=0,1,2,...,n_tbl)  reference table
                    note: tbl_x[] must be ascending or discending sorted
   x                interpolation parameter
   nl               wavelength width of averaging
   xweight          weight table x
   yweight          weight table y
   nweight          number of lines in weight-table

return value        y(x):interpolated value
******************************************************/

#include"tridefine.h"




/*****************************************************
                       interpol.c
            Interpolation ( Lagrange's method )

             K.Kawabata    1995.08.19


arguments          discription

  n_tbl             dimension of tbl_x and tbl_y
 tbl_x[n], tbl_y[n] (n=0,1,2,...,n_tbl)  reference table
                    note: tbl_x[] must be ascending or discending sorted
   x                interpolation parameter
   order            order of interpolation ( 1 < order < 10 )

return value        y(x):interpolated value
******************************************************/

const char *FILH = "fil_";
const char *FILT = ".dat";


double interpol( int n_tbl, double tbl_x[], double tbl_y[], double x, int order){

  int i, j, n0, n1, n2, n_last;
  double xi, xj, y, li, sum;
  double thres = 1.0e-5;
  
  /*** Is the order is proper ? ***/
  if( order < 1 || order > 10 ){
    printf( "order must be between 1 and 10\n" );
    exit(1);
  }
  /*** Is x between tbl_x[1] and tbl_x[n_tbl-1] ? ***/
  /* To keep good precision we can't use both edge of x */
  
/*        if( ( x - tbl_x[1] ) * ( x - tbl_x[n_tbl-1] ) > 0.0 ){ */

  /*** Avoid marume-gosa near edge (981027) ***/

  for( i=0; i<=n_tbl; i++ ){
    if( fabs( tbl_x[i] - x ) < thres*x )
      x = tbl_x[i];
  }
  /*
  if( x < tbl_x[0] && fabs( tbl_x[0] - x ) < 1.0e-5*x )
    x = tbl_x[0];
  if( x > tbl_x[n_tbl] && fabs( tbl_x[n_tbl] - x ) < 1.0e-5*x )
    x = tbl_x[n_tbl];
  */

  if( ( x - tbl_x[0] ) * ( x - tbl_x[n_tbl] ) > 0.0 ){
    printf( "x=%e must be between x[0]=%e and x[n_tbl]=%e\n",\
	    x, tbl_x[0], tbl_x[n_tbl] );
    exit(1);
  }


  /*** search for (x = x_tbl) ***/
  
  n0 = 0;
  n1 = n_tbl;
  while( ( n1 - n0 ) > 1 ){
    n2 = (n0 + n1) / 2.0;
    if( ( x - tbl_x[n0] ) * ( x - tbl_x[n2] ) < 0 )
      n1 = n2;
    else
      n0 = n2;
  }

  /*** select fitting points ***/
  
  if( ( order & 0x01 ) == 0x01 ){
    n0 -= (order - 1)/2;
    n1 += (order - 1)/2;
  }
  else{
    if( fabs( x - tbl_x[n0] ) < fabs( ( tbl_x[n0] - tbl_x[n1] ) / 2 ) ){
      n0 -= order / 2;
      n1 += order / 2 - 1;
    }
    else{
      n0 -= order / 2 - 1;
      n1 += order / 2;
    }
  }
  if( x == tbl_x[n1] ){
    y = tbl_y[n1];
    goto end;
  }
  if( x == tbl_x[n2] ){
    y = tbl_y[n2];
    goto end;
  }
  
  if( (n1 - n0) != order ){
    printf("calculation of order is wrong\n");
    exit(1);
  }
  
  /*** interpolation ***/

  y = 0.0;

  for( i=n0; i<=n1; i++ ){
    xi = tbl_x[i];
    li = 1.0;
    for( j=n0; j<=n1; j++ ){
      if( i != j ){
	xj = tbl_x[j];
	li *= ( x - xj )/( xi - xj );
      }
    }
    y += tbl_y[i] * li;
  }
  
end :
    return( y );
}


float interpolf( int n_tbl, float tbl_x[], float tbl_y[], float x, int order){

  int i, j, n0, n1, n2, n_last;
  double xi, xj, li, sum;
  float y;
  double thres = 1.0e-5;
  
  /*** Is the order is proper ? ***/
  if( order < 1 || order > 10 ){
    printf( "order must be between 1 and 10\n" );
    exit(1);
  }
  /*** Is x between tbl_x[1] and tbl_x[n_tbl-1] ? ***/
  /* To keep good precision we can't use both edge of x */
  
/*        if( ( x - tbl_x[1] ) * ( x - tbl_x[n_tbl-1] ) > 0.0 ){ */

  /*** Avoid marume-gosa near edge (981027) ***/

  for( i=0; i<=n_tbl; i++ ){
    if( fabs( tbl_x[i] - x ) < thres*x )
      x = tbl_x[i];
  }
  /*
  if( x < tbl_x[0] && fabs( tbl_x[0] - x ) < 1.0e-5*x )
    x = tbl_x[0];
  if( x > tbl_x[n_tbl] && fabs( tbl_x[n_tbl] - x ) < 1.0e-5*x )
    x = tbl_x[n_tbl];
  */

  if( ( x - tbl_x[0] ) * ( x - tbl_x[n_tbl] ) > 0.0 ){
    printf( "x=%e must be between x[0]=%e and x[n_tbl]=%e\n",\
	    x, tbl_x[0], tbl_x[n_tbl] );
    exit(1);
  }

  /*** search for (x = x_tbl) ***/
  
  n0 = 0;
  n1 = n_tbl;
  while( ( n1 - n0 ) > 1 ){
    n2 = (n0 + n1) / 2.0;
    if( ( x - tbl_x[n0] ) * ( x - tbl_x[n2] ) < 0 )
      n1 = n2;
    else
      n0 = n2;
  }

  /*** select fitting points ***/
  
  if( ( order & 0x01 ) == 0x01 ){
    n0 -= (order - 1)/2;
    n1 += (order - 1)/2;
  }
  else{
    if( fabs( x - tbl_x[n0] ) < fabs( ( tbl_x[n0] - tbl_x[n1] ) / 2 ) ){
      n0 -= order / 2;
      n1 += order / 2 - 1;
    }
    else{
      n0 -= order / 2 - 1;
      n1 += order / 2;
    }
  }
  if( x == tbl_x[n1] ){
    y = tbl_y[n1];
    goto end;
  }
  if( x == tbl_x[n2] ){
    y = tbl_y[n2];
    goto end;
  }
  
  if( (n1 - n0) != order ){
    printf("calculation of order is wrong\n");
    exit(1);
  }
  
  /*** interpolation ***/

  y = 0.0;

  for( i=n0; i<=n1; i++ ){
    xi = tbl_x[i];
    li = 1.0;
    for( j=n0; j<=n1; j++ ){
      if( i != j ){
	xj = tbl_x[j];
	li *= ( x - xj )/( xi - xj );
      }
    }
    y += tbl_y[i] * li;
  }
  
end :
    return( y );
}

double realcount5b( double c[], double x_low, double x_high, double wl[], double bw[], int n, double lsr[], double sr[], int nsr ){

  int ix_low, ix_high;
  int i, j;
  double count, tcount = 0.0;
  int rswp, rewp;
  double a;
  double lmd, sysres;
  double intwid = 0.0;
  double comwei;

  rswp = ( RSW - SW + DW / 2.0 ) / DW + 1.0 ;
  rewp = ( REW - SW + DW / 2.0 ) / DW + 1.0 ;

  if( x_low >= x_high || ( x_low < ( rswp - 0.5 ) && x_high < ( rswp - 0.5 ) ) || ( x_low > ( rewp + 0.5 ) && x_high > ( rewp + 0.5 ) ) ){
    tcount = 0.0;
  }
  else{
    if( x_low < ( rswp - 0.5 ) )
      x_low = 0.5;
    if( x_high > ( rewp + 0.5 ) )
      x_high = NX2 + 0.5;

    ix_low = x_low + 1.4999999999;
    ix_high = x_high - 0.4999999999;

    if( ix_low <= ix_high ){
      for( j=ix_low; j<=ix_high; j++ ){
        lmd = ( j - 1.0 ) * DW + SW;
	a = interpol( n, wl, bw, lmd, 1 );
        sysres = interpol( nsr, lsr, sr, lmd, 1 );
	comwei = a * sysres;
        count = c[j-1] * comwei;
	tcount += count;
	intwid += comwei;
      }
    }
    if( ( ix_high - ix_low ) > -2 ){
      if( (ix_low - 2) >= ( rswp - 1 ) ){
        lmd = ( ix_low - 1.0 ) * DW + SW;
	a = interpol( n, wl, bw, lmd, 1 );
        sysres = interpol( nsr, lsr, sr, lmd, 1 );
	comwei = a * sysres * ( ( ix_low - 0.5 ) - x_low );
        count = c[ix_low-2] * comwei;
	tcount += count;
	intwid += comwei;
      }
      if( ix_high <= ( rewp - 1 ) ){
        lmd = ( ix_high - 1.0 ) * DW + SW;
	a = interpol( n, wl, bw, lmd, 1 );
        sysres = interpol( nsr, lsr, sr, lmd, 1 );
	comwei = a * sysres * ( x_high - ( ix_high + 0.5 ) );
        count = c[ix_high] * comwei;
	tcount += count;
	intwid += comwei;
      }
    }
    else{
      ix_high = ix_high + 1;
      ix_low = ix_low - 1;
      if( ix_high != ix_low ){
        printf( "Error in realcount\n" );
        return( 0.0 );
      }
      lmd = ( ix_high - 1.0 ) * DW + SW;
      a = interpol( n, wl, bw, lmd, 1 );
      sysres = interpol( nsr, lsr, sr, lmd, 1 );
      comwei = ( x_high - x_low ) * a * sysres;
      count = c[ix_high-1] * comwei;
      tcount += count;
      intwid += comwei;
    }
  }
  if( intwid != 0.0 )
    tcount /= intwid;
  return( tcount );
}

double realcount6b( double c[], double wl1[], double bw1[], int n1, double wl2[], double bw2[], int n2, double lsr[], double sr[], int nsr ){

  int ix_low, ix_high;
  int i, j;
  double a1, a2, lmd, sysres;
  double count, tcount = 0.0;
  double intwid = 0.0;
  double comwei;

  for( i=1; i<=NX2; i++ ){
    lmd = SW + ( i - 1.0 ) * DW;
    if( lmd >= RSW && lmd <= REW ){
      a1 = interpol( n1, wl1, bw1, lmd, 1 );
      a2 = interpol( n2, wl2, bw2, lmd, 1 );
      sysres = interpol( nsr, lsr, sr, lmd, 1 );
      comwei = a1 * a2 * sysres;
      count = c[i-1] * comwei;
      tcount += count;
      intwid += comwei;
    }
  }
  if( intwid != 0.0 )
    tcount /= intwid;
  else
    tcount = -9.9e+30;
  return( tcount );
}

double hbsaverage( int n_tbl, double tbl_x[], double tbl_y[], double x, char *dxc, char *refdir, double xweit[], double yweit[], int nweit, double xphot[], double yphot[], int nphot ){

  FILE *fp;
  char buf[200], last_buf[200];

  double y;
  double xcen=-999.0, xwid=1.0; /* pixel unit */
  int i, j, flag_fil;
  double dwh;

/*
  char filsymb[][30] = { "JU", "JB", "JV", "CR", "CI" };
  char filfile[][30] = { "fil_JU.dat", "fil_JB.dat", "fil_JV.dat", "fil_CR.dat", "fil_CI.dat" };
  int nfil = 5;
*/
  char fn_filter[100];
  double fil_wl[100], fil_bp[100];
  int fil_n;
  double xbinc, xbinw;

  /*** search in which pixel x lies ****/

  //dwh = ( DW / 2.0 ) * 1.0009; /* half width of one pixel in angstrom unit */
  dwh = ( DW / 2.0 ) *1.001; /* half width of one pixel in angstrom unit */

  for( i=1; i<=(n_tbl+1); i++ ){
    if( fabs( tbl_x[i-1] - x ) <= dwh ){
      if( x < tbl_x[i-1] ){
	xcen = i - ( tbl_x[i-1] - x ) / DW;
      }
      else{
	xcen = i + ( x - tbl_x[i-1] ) / DW;
      }
      break;
    }
  }

  if( xcen < -10.0 && i> (n_tbl+1) )
    {
      dwh = ( DW / 2.0 ) *1.002; /* half width of one pixel in angstrom unit */

      for( i=1; i<=(n_tbl+1); i++ ){
	if( fabs( tbl_x[i-1] - x ) <= dwh ){
	  if( x < tbl_x[i-1] ){
	    xcen = i - ( tbl_x[i-1] - x ) / DW;
	  }
	  else{
	    xcen = i + ( x - tbl_x[i-1] ) / DW;
	  }
	  break;
	}
      }
    }

  if( xcen < -10.0 && i> (n_tbl+1) )
    {
      dwh = ( DW / 2.0 ) *1.004; /* half width of one pixel in angstrom unit */

      for( i=1; i<=(n_tbl+1); i++ ){
	if( fabs( tbl_x[i-1] - x ) <= dwh ){
	  if( x < tbl_x[i-1] ){
	    xcen = i - ( tbl_x[i-1] - x ) / DW;
	  }
	  else{
	    xcen = i + ( x - tbl_x[i-1] ) / DW;
	  }
	  break;
	}
      }
    }

  if( xcen < -10.0 && i> (n_tbl+1) )
    {
      dwh = ( DW / 2.0 ) *1.006; /* half width of one pixel in angstrom unit */

      for( i=1; i<=(n_tbl+1); i++ ){
	if( fabs( tbl_x[i-1] - x ) <= dwh ){
	  if( x < tbl_x[i-1] ){
	    xcen = i - ( tbl_x[i-1] - x ) / DW;
	  }
	  else{
	    xcen = i + ( x - tbl_x[i-1] ) / DW;
	  }
	  break;
	}
      }
    }

  if( xcen < -10.0 && i> (n_tbl+1) )
    {
      dwh = ( DW / 2.0 ) *1.008; /* half width of one pixel in angstrom unit */

      for( i=1; i<=(n_tbl+1); i++ ){
	if( fabs( tbl_x[i-1] - x ) <= dwh ){
	  if( x < tbl_x[i-1] ){
	    xcen = i - ( tbl_x[i-1] - x ) / DW;
	  }
	  else{
	    xcen = i + ( x - tbl_x[i-1] ) / DW;
	  }
	  break;
	}
      }
    }

  if( xcen < -10.0 ){
    //printf( "xcen (=%e) is abnormal for x[%d]=%e, tbl_x[0]=%e, tbl_x[%d]=%e dwh=%e.\n", xcen, (i-1), x, tbl_x[0], n_tbl, tbl_x[n_tbl], dwh );
    printf( "xcen (=%e) is abnormal for i=%d, imax=(n_tbl+1)=%d, x=%e, tbl_x[0]=%e, tbl_x[%d]=%e dwh=%e.\n", xcen, (i-1), n_tbl+1, x, tbl_x[0], n_tbl, tbl_x[n_tbl], dwh );
/*    exit( -1 ); */
    return( 0.0 );
  }

/* filter file check & average */

  flag_fil = 0;
  
/* for( j=0; j<=(nfil-1); j++ ){
   if( strcmp( dxc, filsymb[j] ) == 0 ){*/
    if( atoi( dxc ) == 0 ){
        sprintf( fn_filter, "%s%s%s%s", refdir, FILH, dxc, FILT );
/*      sprintf( fn_filter, "%s%s", refdir, filfile[j] ); */
      flag_fil = 1;
/*      break;*/
    }
/*  }*/
  if( flag_fil == 0 ){
    xbinc = ( x - SW ) / DW + 1.0;
    xbinw = atof( dxc ) / DW;
    y = realcount5b( tbl_y, (xbinc-0.5*xbinw), (xbinc+0.5*xbinw), xweit, yweit, nweit, xphot, yphot, nphot );
  }
  else{
    if( ( fp = fopen( fn_filter, "rt" ) ) == NULL ){
      printf( "Filter passband file '%s' not found.\n", fn_filter );
      exit( -1 );
    }
    fil_n = -1;
    while( feof( fp ) == 0 ){
      fgets( buf, 200, fp );
      if( strcmp( buf, last_buf ) == 0 )
	continue;
      fil_n++;
      sscanf( buf, "%lf %lf", &fil_wl[fil_n], &fil_bp[fil_n] );
      strcpy( last_buf, buf );
    }

    fclose( fp );
    if( fil_n == -1 ){
      printf( "Error in reading '%s'\n", fn_filter );
      exit( -1 );
    }
    y = realcount6b( tbl_y, fil_wl, fil_bp, fil_n, xweit, yweit, nweit, xphot, yphot, nphot );
  }

  end :
    return( y );
}
